import java.awt.*; 
import java.awt.event.*;

/*
A demonstration of a non-threaded application with an
unresponsive interface. Clicking on the "Start" button
starts the counter. The button is then labelled "Stop",
and it's supposed to stop the loop via the boolean
variable "stop", but there's no stopping it!
 */

public class CounterNoThread extends Frame
{
    private TextField textWindow;
    private int count;
    private Button toggleButton;
    private boolean stop;
    public static final int HEIGHT = 100;
    public static final int WIDTH = 200;

    public static void main (String [] args)
    {
        CounterNoThread counterWindow = new CounterNoThread();
        counterWindow.setVisible(true);
    }//main

    public CounterNoThread ()
    {
        stop = true;
        count = 0;
        setSize(WIDTH, HEIGHT);

        addWindowListener(new WindowDestroyer());
        setTitle("Non-threaded version");
        setBackground(Color.blue);

        setLayout(new FlowLayout());

        textWindow = new TextField(15);
        add(textWindow);
        textWindow.setText("Starting....");

        toggleButton = new Button("Start");
        toggleButton.addActionListener(new ToggleButtonListener());
        add(toggleButton);

    }//CounterNoThread    

    /*
    This method does the counting. The problem is that when
    actionPerformed() is invoked, it invokes this method WHICH
    NEVER RETURNS because it's an infinite loop! Since go()
    never returns, actionPerformed() never returns from its
    first invocation, so that pressing on the button has no
    effect.
    In the threaded version, the run() method essentially
    plays the role of go().
     */
    public void go()
    {
        while (true)
        {
            try {
                Thread.sleep(50);
            } catch(InterruptedException e) {
                System.err.println("Interrupted.");
            }
            if (!stop)
                textWindow.setText(Integer.toString(count++));
        }//while
    }//go

    /*
     * This inner class is the private listener class for the
     * toggleButton.
     */
    private class ToggleButtonListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e) 
        {
            if (e.getActionCommand().equals("Stop"))
            {
                toggleButton.setLabel("Start");
                stop = true;
            }
            else if (e.getActionCommand().equals("Start"))
            {
                toggleButton.setLabel("Stop");
                stop = false;
                go();
            }
        }//actionPerformed 
    }

}//CounterNoThread


